/***********************************************************
 * $Id: CallbackInvocationContext.java 48 2008-10-29 15:34:24Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 15.10.2008
 * 
 ***********************************************************/

package org.clazzes.remoting;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.rmi.server.UID;
import java.util.Map;

/**
 * An invocation context, which allows callbacks to the originating invoker.
 * 
 * @author wglas
 */
public interface CallbackInvocationContext extends InvocationContext {
    
    /**
     * @param callbackUID The ID of the callback as returned by
     *             {@link RemoteInvoker#registerCallback(InvocationHandler)}. 
     * @param arg The argument, which is passed to the {@link InvocationHandler}.
     * @param metadata The metadata passed to the {@link InvocationHandler}.
     * @return The result of the callback
     * @throws InvocationTargetException If {@link InvocationHandler#invoke(InvocationContext, Object, Map)}
     *                 on the server threw an exception. The exception of the invoker may
     *                 be retrieved by {@link InvocationTargetException#getCause()}.
     * @throws IOException Upon transport errors.
     */
    public Object callBack(UID callbackUID, Object arg, Map<String,Object> metadata)
    throws InvocationTargetException, IOException;

}
