/***********************************************************
 * $Id: InvocationTarget.java 6 2008-10-16 17:24:04Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 15.10.2008
 * 
 ***********************************************************/

package org.clazzes.remoting;

import java.io.IOException;
import java.util.Map;


/**
 * This interface is implemented by servers, which are target for invocation calls.
 * 
 * @author wglas
 */
public interface InvocationTarget {

    /**
     * @param subSystem The name of the subsystem of the handler being registered.
     * @param handler The handler being registered.
     */
    public void registerInvocationHandler(String subSystem, InvocationHandler handler);
    
    /**
     * @param subSystem The subsystem name of the handler being registered.
     */
    public void deregisterInvocationHandler(String subSystem);
    
    /**
     * @param handlers
     */
    public void setInvocationHandlers(Map<String,InvocationHandler> handlers);
    
    /**
     * Shut down all clients and prepare for dismantling the server. 
     */
    public void shutdown();

    /**
     * Start listening for connections.
     * 
     * @throws IOException Upon errors from the network layer.
     */
    public void listen() throws IOException;
}
