/***********************************************************
 * $Id: SocketParameters.java 48 2008-10-29 15:34:24Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 15.10.2008
 * 
 ***********************************************************/

package org.clazzes.remoting;

import java.io.IOException;
import java.net.Socket;

import org.clazzes.remoting.marshal.DefaultMarshalerFactory;
import org.clazzes.remoting.marshal.MarshalerFactory;

/**
 * Common parameters for server and client.
 * 
 * @author wglas
 */
public class SocketParameters {

    protected MarshalerFactory marshalerFactory;
    private boolean tcpNoDelay;

    /**
     * Construct a socket parameters object with the following default parameters:
     * <table>
     *   <tr><td>tcpNoDelay</td><td>true</td></tr>
     * </table>
     */
    public SocketParameters() {
        super();
        this.tcpNoDelay = true;
    }

    /**
     * Apply user-defined setting to the socket. This may involve
     * setting of SSL-parameters in derived classes.
     * 
     * @param socket The socket to prepare for connecting.
     * @throws IOException Upon errors from methods of {@link Socket}.
     */
    public void tuneSocket(Socket socket) throws IOException {
        
        socket.setTcpNoDelay(this.tcpNoDelay);
    }

    /**
     * @return Whether to set TCP_NODELAY on generated sockets.
     * 
     * @see Socket#setTcpNoDelay(boolean)
     */
    public boolean isTcpNoDelay() {
        return this.tcpNoDelay;
    }

    /**
     * @param tcpNoDelay Whether to set TCP_NODELAY on generated sockets.
     * 
     * @see Socket#setTcpNoDelay(boolean)
     */
    public void setTcpNoDelay(boolean tcpNoDelay) {
        this.tcpNoDelay = tcpNoDelay;
    }

    /**
     * @return The {@link MarshalerFactory} to use. If not set by {@link #setMarshalerFactory(MarshalerFactory)},
     *         an instance of {@link DefaultMarshalerFactory} is returned.
     */
    public MarshalerFactory getMarshalerFactory() {
        
        if (this.marshalerFactory == null)
            this.marshalerFactory = new DefaultMarshalerFactory();
            
        return this.marshalerFactory;
    }

    /**
     * @param marshalerFactory The marshaler factory to set.
     */
    public void setMarshalerFactory(MarshalerFactory marshalerFactory) {
        this.marshalerFactory = marshalerFactory;
    }

}