/***********************************************************
 * $Id: InvocationCmd.java 5 2008-10-15 20:38:35Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 15.10.2008
 * 
 ***********************************************************/

package org.clazzes.remoting.cmd;

import java.util.Map;

import org.clazzes.remoting.InvocationHandler;

/**
 * @author wglas
 *
 */
public class InvocationCmd extends Cmd {

    private static final long serialVersionUID = 2249775601424926139L;

    private final String subSystem;
    private final Object arg;
    private final Map<String,Object> metadata;
    
    /**
     * @param subSystem The subSystem being called.
     * @param arg The argument to the call.
     * @param metadata The metadata passed to the call.
     */
    public InvocationCmd(String subSystem, Object arg,
            Map<String, Object> metadata) {
        
        super();
        this.subSystem = subSystem;
        this.arg = arg;
        this.metadata = metadata;
    }

    /**
     * @return The subsystem being called.
     */
    public String getSubSystem() {
        return this.subSystem;
    }

    /**
     * @return The argument to the subsystem.
     */
    public Object getArg() {
        return this.arg;
    }

    /**
     * @return The metadata passed to the {@link InvocationHandler}
     */
    public Map<String, Object> getMetadata() {
        return this.metadata;
    }

    /* (non-Javadoc)
     * @see org.clazzes.remoting.cmd.Cmd#accept(org.clazzes.remoting.cmd.CmdVisitor)
     */
    @Override
    public void accept(CmdVisitor visitor) {
       
        visitor.visit(this);
    }
}
