/***********************************************************
 * $Id: Marshaler.java 12 2008-10-18 08:21:27Z wglas $
 * 
 * remoting framework of the clazzes project.
 * http://www.clazzes.org
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Created: 15.10.2008
 * 
 ***********************************************************/

package org.clazzes.remoting.marshal;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

/**
 * A marshaller, which handles the serialization and deserialization of objects.
 * 
 * @author wglas
 */
public interface Marshaler {

    /**
     * Set up the marshaler for operation. Implementations must guarantee,
     * that  both, the input and the ouput stream are closed, when an excpetion is thrown.
     * 
     * @param os The output stream to write to.
     * @param is The input stream to read from.
     * @param inputClassLoader The ClassLoader to use for deserialized instances.
     * @throws IOException Upon errors setting up the streams.
     */
    public void setup(OutputStream os, InputStream is, ClassLoader inputClassLoader) throws IOException;
    
    /**
     * @return The next object on the input stream.
     * @throws ClassNotFoundException If the underlying calss loader was unable
     *               to find the class of a serialized object.
     * @throws IOException Upon transport errors.
     */
    public Object readObject() throws ClassNotFoundException, IOException;
        
    /**
     * @param obj The object to write.
     * @throws IOException Upon transport errors.
     */
    public void writeObject(Object obj) throws IOException;
    
    /**
     * Reset the state of the serializer. This incurs the cleanup of the
     * object cache, which ensures, that the same instance of a class is serialized
     * as reference rather than being serialized a second time.
     * @throws IOException  Upon transport errors.
     */
    public void reset() throws IOException;
    
    /**
     * Close the underlying transport and prepare the marshaler for
     * calling {@link #setup(OutputStream, InputStream, ClassLoader)} again.
     * 
     * This method should be implemented in a fashion that is tolerant for an
     * incomplete setup of the marshaler. It should even be possible to call
     * close() twice on a single marshaler object.
     */
    public void close();
}
